@@ -67,8 +67,13 @@ typedef struct {
   GdkGLContextProgram texture_rect_quad_program;
 
   GdkGLContextProgram *current_program;
+
+  guint is_legacy : 1;
 } GdkGLContextPaintData;
 
+void                    gdk_gl_context_set_is_legacy            (GdkGLContext    *context,
+                                                                 gboolean         is_legacy);
+
 void                    gdk_gl_context_upload_texture           (GdkGLContext    *context,
                                                                  cairo_surface_t *image_surface,
                                                                  int              width,
