@@ -40,7 +40,9 @@
  * may be in the top 8 bits of the pixel, or may be in the lower 4 bits.
  *
  * There are several standard visuals. The visual returned by
- * gdk_screen_get_system_visual() is the system’s default visual.
+ * gdk_screen_get_system_visual() is the system’s default visual, and
+ * the visual returned by gdk_screen_get_rgba_visual() should be used for
+ * creating windows with an alpha channel.
  *
  * A number of functions are provided for determining the “best” available
  * visual. For the purposes of making this determination, higher bit depths
@@ -349,6 +351,44 @@ gdk_visual_get_bits_per_rgb (GdkVisual *visual)
   return visual->bits_per_rgb;
 }
 
+static void
+gdk_visual_get_pixel_details (GdkVisual *visual,
+                              gulong     pixel_mask,
+                              guint32   *mask,
+                              gint      *shift,
+                              gint      *precision)
+{
+  gulong m = 0;
+  gint s = 0;
+  gint p = 0;
+
+  if (pixel_mask != 0)
+    {
+      m = pixel_mask;
+      while (!(m & 0x1))
+        {
+          s++;
+          m >>= 1;
+        }
+
+      m = pixel_mask;
+      while (m & 0x1)
+        {
+          p++;
+          m >>= 1;
+        }
+    }
+
+  if (mask)
+    *mask = pixel_mask;
+
+  if (shift)
+    *shift = s;
+
+  if (precision)
+    *precision = p;
+}
+
 /**
  * gdk_visual_get_red_pixel_details:
  * @visual: A #GdkVisual
@@ -372,14 +412,7 @@ gdk_visual_get_red_pixel_details (GdkVisual *visual,
 {
   g_return_if_fail (GDK_IS_VISUAL (visual));
 
-  if (mask)
-    *mask = visual->red_mask;
-
-  if (shift)
-    *shift = visual->red_shift;
-
-  if (precision)
-    *precision = visual->red_prec;
+  gdk_visual_get_pixel_details (visual, visual->red_mask, mask, shift, precision);
 }
 
 /**
@@ -405,14 +438,7 @@ gdk_visual_get_green_pixel_details (GdkVisual *visual,
 {
   g_return_if_fail (GDK_IS_VISUAL (visual));
 
-  if (mask)
-    *mask = visual->green_mask;
-
-  if (shift)
-    *shift = visual->green_shift;
-
-  if (precision)
-    *precision = visual->green_prec;
+  gdk_visual_get_pixel_details (visual, visual->green_mask, mask, shift, precision);
 }
 
 /**
@@ -438,14 +464,7 @@ gdk_visual_get_blue_pixel_details (GdkVisual *visual,
 {
   g_return_if_fail (GDK_IS_VISUAL (visual));
 
-  if (mask)
-    *mask = visual->blue_mask;
-
-  if (shift)
-    *shift = visual->blue_shift;
-
-  if (precision)
-    *precision = visual->blue_prec;
+  gdk_visual_get_pixel_details (visual, visual->blue_mask, mask, shift, precision);
 }
 
 /**
