@@ -538,7 +538,7 @@ static TranslationEntry translations[] = {
   { "org.gnome.desktop.interface", "cursor-blink-timeout", "gtk-cursor-blink-timeout", G_TYPE_INT, { .i = 3600 } },
   { "org.gnome.desktop.interface", "gtk-im-module", "gtk-im-module", G_TYPE_STRING, { .s = "simple" } },
   { "org.gnome.desktop.interface", "enable-animations", "gtk-enable-animations", G_TYPE_BOOLEAN, { .b = TRUE } },
-  { "org.gnome.settings-daemon.peripherals.mouse", "double-click", "gtk-double-click-time", G_TYPE_INT, { .i = 250 } },
+  { "org.gnome.settings-daemon.peripherals.mouse", "double-click", "gtk-double-click-time", G_TYPE_INT, { .i = 400 } },
   { "org.gnome.settings-daemon.peripherals.mouse", "drag-threshold", "gtk-dnd-drag-threshold", G_TYPE_INT, {.i = 8 } },
   { "org.gnome.desktop.sound", "theme-name", "gtk-sound-theme-name", G_TYPE_STRING, { .s = "freedesktop" } },
   { "org.gnome.desktop.sound", "event-sounds", "gtk-enable-event-sounds", G_TYPE_BOOLEAN, { .b = TRUE } },
@@ -555,6 +555,7 @@ static TranslationEntry translations[] = {
   { "org.gnome.desktop.wm.preferences", "action-double-click-titlebar", "gtk-titlebar-double-click", G_TYPE_STRING, { .s = "toggle-maximize" } },
   { "org.gnome.desktop.wm.preferences", "action-middle-click-titlebar", "gtk-titlebar-middle-click", G_TYPE_STRING, { .s = "none" } },
   { "org.gnome.desktop.wm.preferences", "action-right-click-titlebar", "gtk-titlebar-right-click", G_TYPE_STRING, { .s = "menu" } },
+  { "org.gnome.desktop.a11y", "always-show-text-caret", "gtk-keynav-use-caret", G_TYPE_BOOLEAN, { .b = FALSE } }
 };
 
 static TranslationEntry *
@@ -631,7 +632,7 @@ init_settings (GdkScreen *screen)
       if (g_hash_table_lookup (screen_wayland->settings, (gpointer)translations[i].schema) != NULL)
         continue;
 
-      schema = g_settings_schema_source_lookup (source, translations[i].schema, FALSE);
+      schema = g_settings_schema_source_lookup (source, translations[i].schema, TRUE);
       if (schema != NULL)
         {
           settings = g_settings_new_full (schema, NULL, NULL);
@@ -646,9 +647,9 @@ init_settings (GdkScreen *screen)
 }
 
 static void
-gtk_shell_handle_capabilities (void             *data,
-			       struct gtk_shell *shell,
-			       uint32_t          capabilities)
+gtk_shell_handle_capabilities (void              *data,
+                               struct gtk_shell1 *shell,
+                               uint32_t           capabilities)
 {
   GdkScreen *screen = data;
   GdkWaylandScreen *screen_wayland = GDK_WAYLAND_SCREEN (data);
@@ -660,7 +661,7 @@ gtk_shell_handle_capabilities (void             *data,
   notify_setting (screen, "gtk-shell-shows-desktop");
 }
 
-struct gtk_shell_listener gdk_screen_gtk_shell_listener = {
+struct gtk_shell1_listener gdk_screen_gtk_shell_listener = {
   gtk_shell_handle_capabilities
 };
 
@@ -669,7 +670,9 @@ _gdk_wayland_screen_set_has_gtk_shell (GdkScreen *screen)
 {
   GdkWaylandDisplay *wayland_display = GDK_WAYLAND_DISPLAY (GDK_WAYLAND_SCREEN (screen)->display);
 
-  gtk_shell_add_listener (wayland_display->gtk_shell, &gdk_screen_gtk_shell_listener, screen);
+  gtk_shell1_add_listener (wayland_display->gtk_shell,
+                           &gdk_screen_gtk_shell_listener,
+                           screen);
 }
 
 static void
@@ -763,7 +766,7 @@ set_decoration_layout_from_entry (GdkScreen        *screen,
 static gboolean
 set_capability_setting (GdkScreen                 *screen,
                         GValue                    *value,
-                        enum gtk_shell_capability  test)
+                        enum gtk_shell1_capability test)
 {
   GdkWaylandScreen *wayland_screen = GDK_WAYLAND_SCREEN (screen);
 
@@ -792,13 +795,16 @@ gdk_wayland_screen_get_setting (GdkScreen   *screen,
    }
 
   if (strcmp (name, "gtk-shell-shows-app-menu") == 0)
-    return set_capability_setting (screen, value, GTK_SHELL_CAPABILITY_GLOBAL_APP_MENU);
+    return set_capability_setting (screen, value,
+                                   GTK_SHELL1_CAPABILITY_GLOBAL_APP_MENU);
 
   if (strcmp (name, "gtk-shell-shows-menubar") == 0)
-    return set_capability_setting (screen, value, GTK_SHELL_CAPABILITY_GLOBAL_MENU_BAR);
+    return set_capability_setting (screen, value,
+                                   GTK_SHELL1_CAPABILITY_GLOBAL_MENU_BAR);
 
   if (strcmp (name, "gtk-shell-shows-desktop") == 0)
-    return set_capability_setting (screen, value, GTK_SHELL_CAPABILITY_DESKTOP_ICONS);
+    return set_capability_setting (screen, value,
+                                   GTK_SHELL1_CAPABILITY_DESKTOP_ICONS);
 
   if (strcmp (name, "gtk-dialogs-use-header") == 0)
     {
