@@ -161,27 +161,28 @@ static guint debug_flags = 0;              /* Global GTK debug flag */
 
 #ifdef G_ENABLE_DEBUG
 static const GDebugKey gtk_debug_keys[] = {
-  {"misc", GTK_DEBUG_MISC},
-  {"plugsocket", GTK_DEBUG_PLUGSOCKET},
-  {"text", GTK_DEBUG_TEXT},
-  {"tree", GTK_DEBUG_TREE},
-  {"updates", GTK_DEBUG_UPDATES},
-  {"keybindings", GTK_DEBUG_KEYBINDINGS},
-  {"multihead", GTK_DEBUG_MULTIHEAD},
-  {"modules", GTK_DEBUG_MODULES},
-  {"geometry", GTK_DEBUG_GEOMETRY},
-  {"icontheme", GTK_DEBUG_ICONTHEME},
-  {"printing", GTK_DEBUG_PRINTING},
-  {"builder", GTK_DEBUG_BUILDER},
-  {"size-request", GTK_DEBUG_SIZE_REQUEST},
-  {"no-css-cache", GTK_DEBUG_NO_CSS_CACHE},
-  {"baselines", GTK_DEBUG_BASELINES},
-  {"pixel-cache", GTK_DEBUG_PIXEL_CACHE},
-  {"no-pixel-cache", GTK_DEBUG_NO_PIXEL_CACHE},
-  {"interactive", GTK_DEBUG_INTERACTIVE},
-  {"touchscreen", GTK_DEBUG_TOUCHSCREEN},
-  {"actions", GTK_DEBUG_ACTIONS},
-  {"resize", GTK_DEBUG_RESIZE}
+  { "misc", GTK_DEBUG_MISC },
+  { "plugsocket", GTK_DEBUG_PLUGSOCKET },
+  { "text", GTK_DEBUG_TEXT },
+  { "tree", GTK_DEBUG_TREE },
+  { "updates", GTK_DEBUG_UPDATES },
+  { "keybindings", GTK_DEBUG_KEYBINDINGS },
+  { "multihead", GTK_DEBUG_MULTIHEAD },
+  { "modules", GTK_DEBUG_MODULES },
+  { "geometry", GTK_DEBUG_GEOMETRY },
+  { "icontheme", GTK_DEBUG_ICONTHEME },
+  { "printing", GTK_DEBUG_PRINTING} ,
+  { "builder", GTK_DEBUG_BUILDER },
+  { "size-request", GTK_DEBUG_SIZE_REQUEST },
+  { "no-css-cache", GTK_DEBUG_NO_CSS_CACHE },
+  { "baselines", GTK_DEBUG_BASELINES },
+  { "pixel-cache", GTK_DEBUG_PIXEL_CACHE },
+  { "no-pixel-cache", GTK_DEBUG_NO_PIXEL_CACHE },
+  { "interactive", GTK_DEBUG_INTERACTIVE },
+  { "touchscreen", GTK_DEBUG_TOUCHSCREEN },
+  { "actions", GTK_DEBUG_ACTIONS },
+  { "resize", GTK_DEBUG_RESIZE },
+  { "layout", GTK_DEBUG_LAYOUT }
 };
 #endif /* G_ENABLE_DEBUG */
 
@@ -693,6 +694,7 @@ static void
 default_display_notify_cb (GdkDisplayManager *dm)
 {
   _gtk_accessibility_init ();
+  gtk_set_debug_flags (debug_flags);
 }
 
 static void
@@ -795,6 +797,24 @@ post_parse_hook (GOptionContext *context,
   return TRUE;
 }
 
+guint
+gtk_get_display_debug_flags (GdkDisplay *display)
+{
+  if (display)
+    return GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (display), "gtk-debug-flags"));
+  else
+    return debug_flags;
+}
+
+void
+gtk_set_display_debug_flags (GdkDisplay *display,
+                             guint       flags)
+{
+  if (display)
+    g_object_set_data (G_OBJECT (display), "gtk-debug-flags", GUINT_TO_POINTER (flags));
+  else
+    debug_flags = flags;
+}
 
 /**
  * gtk_get_debug_flags:
@@ -809,7 +829,7 @@ post_parse_hook (GOptionContext *context,
 guint
 gtk_get_debug_flags (void)
 {
-  return debug_flags;
+  return gtk_get_display_debug_flags (gdk_display_get_default ());
 }
 
 /**
@@ -820,7 +840,7 @@ gtk_get_debug_flags (void)
 void
 gtk_set_debug_flags (guint flags)
 {
-  debug_flags = flags;
+  gtk_set_display_debug_flags (gdk_display_get_default (), flags);
 }
 
 gboolean
@@ -831,7 +851,7 @@ gtk_simulate_touchscreen (void)
   if (test_touchscreen == 0)
     test_touchscreen = g_getenv ("GTK_TEST_TOUCHSCREEN") != NULL ? 1 : -1;
 
-  return test_touchscreen > 0 || (debug_flags & GTK_DEBUG_TOUCHSCREEN) != 0;
+  return test_touchscreen > 0 || (gtk_get_debug_flags () & GTK_DEBUG_TOUCHSCREEN) != 0;
  }
 
 /**
@@ -1026,7 +1046,7 @@ gtk_init_check (int    *argc,
 
   ret = GDK_PRIVATE_CALL (gdk_display_open_default) () != NULL;
 
-  if (debug_flags & GTK_DEBUG_INTERACTIVE)
+  if (gtk_get_debug_flags () & GTK_DEBUG_INTERACTIVE)
     gtk_window_set_interactive_debugging (TRUE);
 
   return ret;
